package cn.com.qmth.print.manage.controller;

import cn.com.qmth.print.manage.entity.ExamEntity;
import cn.com.qmth.print.manage.service.ExamService;
import cn.com.qmth.print.manage.service.PmSession;
import cn.com.qmth.print.manage.service.query.ExamQuery;
import cn.com.qmth.print.manage.utils.PathUtil;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.StatusException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.util.Objects;

/**
 * @Date: 2021/11/16.
 */
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/exam")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class ExamController extends BaseController {

    @Autowired
    private ExamService examService;

    /**
     * 批次分页查询
     *
     * @param query
     * @return
     */
    @RequestMapping(value = "/page", method = RequestMethod.POST)
    public Object page(@Validated ExamQuery query) {
        return examService.pageQuery(query);
    }

    /**
     * 批次查询
     *
     * @return
     */
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Object list() {
        return examService.listByOrgId();
    }

    /**
     * 新增/修改批次
     *
     * @param exam
     * @return
     */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Object save(@RequestAttribute PmSession accessEntity, ExamEntity exam) {
        Long userId = getAccessUserId(accessEntity);
        exam.setCreatorId(userId);
        exam.setUpdaterId(userId);
        return examService.saveExam(exam);
    }

    /**
     * 批次查询
     *
     * @return
     */
    @RequestMapping(value = "/listByChecker", method = RequestMethod.POST)
    public Object listByChecker(@RequestAttribute PmSession accessEntity) {
        Long userId = getAccessUserId(accessEntity);
        return examService.listByUserId(userId);
    }

    @GetMapping("/template")
    public void getImportTemplate() {
        String resourcePath = PathUtil.getResoucePath("importtemplates/examImportTemplate.xlsx");
        if (Objects.isNull(resourcePath)){
            throw new StatusException("未找到模板路径");
        }
        exportFile("学校导入模板.xlsx", new File(resourcePath));
    }

    /**
     * 删除考试批次
     *
     * @param accessEntity 鉴权
     * @param examId       考试id
     */
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public void delete(@RequestAttribute PmSession accessEntity, @RequestParam Long examId) {
        examService.deleteExam(examId);
    }

//    @RequestMapping(value = "/e", method = RequestMethod.POST)
//    public void delete(@RequestAttribute PmSession accessEntity, @RequestParam Long examId) {
//        examService.deleteExam(examId);
//    }
}
