package cn.com.qmth.print.manage.service.impl;

import cn.com.qmth.print.manage.dao.ExamDao;
import cn.com.qmth.print.manage.dao.ExamStudentDao;
import cn.com.qmth.print.manage.dto.ExamStudentExportDTO;
import cn.com.qmth.print.manage.dto.ExamStudentImportDTO;
import cn.com.qmth.print.manage.dto.StudentDTO;
import cn.com.qmth.print.manage.entity.CheckRecordEntity;
import cn.com.qmth.print.manage.entity.ExamEntity;
import cn.com.qmth.print.manage.entity.ExamStudentEntity;
import cn.com.qmth.print.manage.enums.GroupType;
import cn.com.qmth.print.manage.enums.RecordStatus;
import cn.com.qmth.print.manage.service.BreakRecordService;
import cn.com.qmth.print.manage.service.CheckRecordService;
import cn.com.qmth.print.manage.service.ExamStudentService;
import cn.com.qmth.print.manage.service.query.ExamStudentQuery;
import cn.com.qmth.print.manage.utils.Zip4jUtil;
import cn.com.qmth.print.manage.utils.excel.ExcelError;
import cn.com.qmth.print.manage.utils.excel.ExcelReader;
import cn.com.qmth.print.manage.utils.excel.ExportExcel;
import cn.hutool.core.io.FileUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.StatusException;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

@Service
public class ExamStudentServiceImpl extends ServiceImpl<ExamStudentDao, ExamStudentEntity> implements
        ExamStudentService {

    @Resource
    private CheckRecordService checkRecordService;

    @Resource
    private BreakRecordService breakRecordService;

    @Resource
    private ExamDao examDao;

    @Override
    public ExamStudentEntity findByExamIdAndExamNumber(Long examId, String examNumber) {
        // TODO: 2023/10/20 校验更改逻辑 
        if (examId == null) {
            throw new StatusException("examId不能为空");
        }
        if (StringUtils.isEmpty(examNumber)) {
            throw new StatusException("准考证号不能为空");
        }
        QueryWrapper<ExamStudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ExamStudentEntity> lw = wrapper.lambda();
        lw.eq(ExamStudentEntity::getExamId, examId);
        lw.eq(ExamStudentEntity::getExamNumber, examNumber);
        ExamStudentEntity entity = this.getOne(wrapper);
        return entity;
    }

    @Override
    public Long findSortNoByExamIdAndGroupTypeAndSort(Long examId, GroupType type, String groupName, boolean isAse) {
        QueryWrapper<ExamStudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ExamStudentEntity> lw = wrapper.lambda();
        lw.eq(ExamStudentEntity::getExamId, examId);
        if (GroupType.COURSE.equals(type)) {
            lw.eq(ExamStudentEntity::getCourseCode, groupName);
        } else if (GroupType.EXAM_SITE.equals(type)) {
            lw.eq(ExamStudentEntity::getExamSite, groupName);
        } else if (GroupType.EXAM_ROOM.equals(type)) {
            lw.eq(ExamStudentEntity::getExamRoom, groupName);
        }
        if (isAse) {
            wrapper.orderByAsc("sort_no");
        } else {
            wrapper.orderByDesc("sort_no");
        }
        wrapper.last("LIMIT 1");
        ExamStudentEntity entity = this.getOne(wrapper);
        return entity.getSortNo();
    }

    @Override
    public List<ExamStudentEntity> listByExamId(Long examId) {
        QueryWrapper<ExamStudentEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudentEntity::getExamId, examId);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<ExamStudentEntity> pageStudent(ExamStudentQuery query) {
        QueryWrapper<ExamStudentEntity> queryWrapper = new QueryWrapper<>();
        if (query.getExamId() == null) {
            throw new StatusException("请选择批次");
        }
        queryWrapper.lambda().eq(ExamStudentEntity::getExamId, query.getExamId());
        if (StringUtils.isNotBlank(query.getExamNumber())) {
            queryWrapper.lambda().like(ExamStudentEntity::getExamNumber, "%" + query.getExamNumber() + "%");
        }
        if (StringUtils.isNotBlank(query.getCourseCode())) {
            queryWrapper.lambda().like(ExamStudentEntity::getCourseCode, "%" + query.getCourseCode() + "%");
        }
        if (StringUtils.isNotBlank(query.getExamSite())) {
            queryWrapper.lambda().like(ExamStudentEntity::getExamSite, "%" + query.getExamSite() + "%");
        }
        if (StringUtils.isNotBlank(query.getExamRoom())) {
            queryWrapper.lambda().like(ExamStudentEntity::getExamRoom, "%" + query.getExamRoom() + "%");
        }
        Page page = new Page(query.getPageNumber(), query.getPageSize());
        return this.page(page, queryWrapper);
    }

    @Override
    public List<ExcelError> importStudents(Long examId, MultipartFile file) throws IOException {
        ExamEntity examEntity = examDao.selectById(examId);
        List<ExamStudentEntity> studentList = new ArrayList<>();
        ExcelReader excelReader = new ExcelReader(StudentDTO.class);
        AtomicLong aLong = new AtomicLong(1L);
        List<ExcelError> excelErrors = excelReader.reader(file.getInputStream(), obj -> {
            try {
                StudentDTO dto = (StudentDTO) obj;
                ExamStudentEntity examStudentEntity = new ExamStudentEntity();
                examStudentEntity.setExamId(examId);
                examStudentEntity.setOrgId(examEntity.getOrgId());
                examStudentEntity.setExamNumber(dto.getExamNumber());
                examStudentEntity.setStudentCode(dto.getStudentCode());
                examStudentEntity.setName(dto.getName());
                examStudentEntity.setCourseCode(dto.getCourseCode().concat("_").concat(dto.getCourseName()));
                examStudentEntity.setExamSite(dto.getExamSite());
                examStudentEntity.setExamRoom(dto.getExamRoom());
                examStudentEntity.setSortNo(aLong.getAndIncrement());
                examStudentEntity.setCreateTime(new Date());
                examStudentEntity.setUpdateTime(new Date());
                studentList.add(examStudentEntity);
                return null;
            } catch (RuntimeException e) {
                // 手动回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                ExcelError excelError = new ExcelError();
                excelError.setExcelErrorType(e.getMessage());
                return excelError;
            }
        });
        String errors = errorsString(excelErrors);
        if (errors.length() > 0) {
            throw new StatusException(errors);
        }

        if (!CollectionUtils.isEmpty(studentList)) {
            insertData(examEntity, studentList);
        }
        return excelErrors;
    }

    @Transactional
    @Override
    public Object analyzeZipAndImportStudents(Long examId ,MultipartFile file) throws IOException {
        String zipPath = "E:\\file\\studentImport2.zip";
        String dirPath = "E:\\file\\studentImport2";
//        File zip = new File("E:\\file\\studentImport2.zip");
//        File zipDir = new File("E:\\file\\studentImport2");
        Zip4jUtil.unzipEncryptFile(zipPath,dirPath,"qmth!@#");
//        File zip = new File("E:\\file\\studentImport2.zip");
        File zipDir = new File("E:\\file\\studentImport2");
        File[] firstDirArr = zipDir.listFiles();
        if (Objects.isNull(firstDirArr) || firstDirArr.length != 1) {
            throw new StatusException("文件解析失败：未找到一级目录");
        }
        List<File> studentFileList = Arrays.stream(Objects.requireNonNull(firstDirArr[0].listFiles()))
                .filter(e -> e.getName().endsWith("xlsx"))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(studentFileList)) {
            throw new StatusException("文件解析失败：未找到导入的excel文件");
        }
        if (studentFileList.size() != 1) {
            throw new StatusException("文件解析失败：存在多个excel文件");
        }
        File studentExcel = studentFileList.get(0);

        ExamEntity examEntity = examDao.selectById(examId);
        List<ExamStudentEntity> studentList = new ArrayList<>();
        ExcelReader excelReader = new ExcelReader(ExamStudentImportDTO.class);
        AtomicLong sort = new AtomicLong(1L);
        List<ExcelError> excelErrors = excelReader.reader(FileUtil.getInputStream(studentExcel), obj -> {
            try {
                ExamStudentImportDTO dto = (ExamStudentImportDTO) obj;
                ExamStudentEntity examStudentEntity = new ExamStudentEntity();
                examStudentEntity.setExamId(examId);
                examStudentEntity.setOrgId(examEntity.getOrgId());
                examStudentEntity.setExamNumber(dto.getExamNumber());
                examStudentEntity.setStudentCode(dto.getStudentCode());
                examStudentEntity.setName(dto.getStudentName());
                examStudentEntity.setCourseCode(dto.getCourseCode());
                examStudentEntity.setCourseName(dto.getCourseName());
                examStudentEntity.setExamSite(dto.getExamPlaceName());
                examStudentEntity.setSortNo(sort.getAndIncrement());
                examStudentEntity.setExamUnit(dto.getExamUnit());
                examStudentEntity.setCreateTime(new Date());
                examStudentEntity.setUpdateTime(new Date());
                studentList.add(examStudentEntity);
                return null;
            } catch (RuntimeException e) {
                // 手动回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                ExcelError excelError = new ExcelError();
                excelError.setExcelErrorType(e.getMessage());
                return excelError;
            }
        });

        String errors = errorsString(excelErrors);
        if (errors.length() > 0) {
            throw new StatusException(errors);
        }

        if (!CollectionUtils.isEmpty(studentList)) {
            insertData(examEntity, studentList);
        }
        return excelErrors;
    }

    @Override
    public void exportExamStudent(HttpServletResponse response, Long examId) throws IOException {
        List<ExamStudentExportDTO> examStudentExportDTOList = this.baseMapper.findExportData(examId);
        new ExportExcel("考生数据", ExamStudentExportDTO.class).setDataList(examStudentExportDTOList).write(response, "")
                .dispose();
    }

    @Override
    public void deleteByExamId(Long examId) {
        UpdateWrapper<ExamStudentEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(ExamStudentEntity::getExamId, examId);
        this.remove(updateWrapper);
    }

    @Transactional
    public void insertData(ExamEntity examEntity, List<ExamStudentEntity> studentList) {
        // 有校验数据不能删除
        if (checkRecordService.count(new QueryWrapper<CheckRecordEntity>().lambda()
                .eq(CheckRecordEntity::getExamId,examEntity.getId())
                .ne(CheckRecordEntity::getStatus,RecordStatus.NONE)) > 0){
            throw new StatusException("已有校验数据，不可重复导入");
        }

        // 删除考生表
        this.deleteByExamId(examEntity.getId());
        // 删除校验记录
        checkRecordService.deleteByExamId(examEntity.getId());
        // 删除中断记录
        breakRecordService.deleteByExamId(examEntity.getId());

        // 添加考生
        List<ExamStudentEntity> data = new ArrayList<>();
        for (ExamStudentEntity stu : studentList) {
            if (data.size() == 2000) {
                this.saveBatch(data);
                data.clear();
            }
            data.add(stu);
        }
        // 将剩下的数据也导入
        if (!data.isEmpty()) {
            this.saveBatch(data);
        }
        Map<String, List<ExamStudentEntity>> groupCollection = null;

        if (examEntity.getGroupType().equals(GroupType.COURSE)) {
            groupCollection = studentList.stream().collect(Collectors.groupingBy(ExamStudentEntity::getCourseCode));
        } else if (examEntity.getGroupType().equals(GroupType.EXAM_SITE)) {
            groupCollection = studentList.stream().collect(Collectors.groupingBy(ExamStudentEntity::getExamSite));
        } else if (examEntity.getGroupType().equals(GroupType.EXAM_ROOM)) {
            groupCollection = studentList.stream().collect(Collectors.groupingBy(ExamStudentEntity::getExamRoom));
        }
        // 添加校验数据

        List<CheckRecordEntity> checkRecordEntities = new ArrayList<>();
        for (Map.Entry<String, List<ExamStudentEntity>> entry : groupCollection.entrySet()) {
            CheckRecordEntity checkRecordEntity = new CheckRecordEntity();
            checkRecordEntity.setOrgId(examEntity.getOrgId());
            checkRecordEntity.setExamId(examEntity.getId());
            checkRecordEntity.setGroupType(examEntity.getGroupType());
            checkRecordEntity.setGroupName(entry.getKey());
            checkRecordEntity.setStatus(RecordStatus.NONE);
            checkRecordEntity.setCreateTime(new Date());
            checkRecordEntity.setUpdateTime(new Date());
            checkRecordEntities.add(checkRecordEntity);
        }
        checkRecordService.saveBatch(checkRecordEntities);
    }

    /**
     * 拼接导入异常信息
     *
     * @param excelErrors
     * @return
     */
    private String errorsString(List<ExcelError> excelErrors) {
        StringJoiner sj = new StringJoiner(";");
        if (!excelErrors.isEmpty() && excelErrors.size() > 0) {
            int forint = excelErrors.size() < 10 ? excelErrors.size() : 9;
            for (int i = 0; i < forint; i++) {
                ExcelError excelError = excelErrors.get(i);
                StringBuffer sb = new StringBuffer();
                sb.append("第").append(excelError.getRow()).append("行,").append(excelError.getExcelErrorType());
                sj.add(sb.toString());
            }
        }
        return sj.toString();
    }
}
